@@ -22,7 +22,7 @@
 *
 * \section intro_sec Introduction
 *
-* libqalculate is math libary for expression evaluation with units, variables and functions support and CAS functionality.
+* libqalculate is a math library for expression evaluation with units, variables and functions support and CAS functionality.
 *
 * The main parts of the library is the almighty Calculator class, the MathStructure class for mathematical expressions and classes for objects in an expression,
 * mostly of the class Numbers and sub classes of ExpressionItem.
@@ -35,7 +35,7 @@
 * CALCULATOR->loadLocalDefinitions();
 * cout << CALCULATOR->calculateAndPrint("1 + 1", 2000) << endl;\endcode
 * In the above example, the calculation is terminated after two seconds (2000 ms), if it is not finished before then.
-* Applications using localized numbers should first call Calculalor::unlocalizeExpression() on the expression.
+* Applications using localized numbers should first call Calculator::unlocalizeExpression() on the expression.
 *
 * A less simple application might calculate and output the expression separately.
 * \code
@@ -49,9 +49,9 @@
 * \code
 * PrintOptions po;
 * string result_str = CALCULATOR->print(result, 2000, po);\endcode
-* Alternatively MathStructure::format() followed by MathStructure::print() can be used, whithout the possiblity to specify a time limit.
+* Alternatively MathStructure::format() followed by MathStructure::print() can be used, without the possibility to specify a time limit.
 *
-* Central to the flexiblity of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
+* Central to the flexibility of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
 *
 * \section usage_sec Using the library
 *
@@ -102,13 +102,13 @@ struct PlotParameters {
 	float y_max;
 	/// Maximum x-axis value.
 	float x_max;
-	/// If a logarithimic scale shall be used for the y-axis. Default: false
+	/// If a logarithmic scale shall be used for the y-axis. Default: false
 	bool y_log;
-	/// If a logarithimic scale shall be used for the x-axis. Default: false
+	/// If a logarithmic scale shall be used for the x-axis. Default: false
 	bool x_log;
-	/// Logarithimic base for the y-axis. Default: 10
+	/// Logarithmic base for the y-axis. Default: 10
 	int y_log_base;
-	/// Logarithimic base for the x-axis. Default: 10
+	/// Logarithmic base for the x-axis. Default: 10
 	int x_log_base;
 	/// If  a grid shall be shown in the plot. Default: false
 	bool grid;
@@ -145,6 +145,20 @@ typedef enum {
 	MESSAGE_ERROR
 } MessageType;
 
+typedef enum {
+	AUTOMATIC_FRACTION_OFF,
+	AUTOMATIC_FRACTION_SINGLE,
+	AUTOMATIC_FRACTION_AUTO,
+	AUTOMATIC_FRACTION_DUAL
+} AutomaticFractionFormat;
+
+typedef enum {
+	AUTOMATIC_APPROXIMATION_OFF,
+	AUTOMATIC_APPROXIMATION_SINGLE,
+	AUTOMATIC_APPROXIMATION_AUTO,
+	AUTOMATIC_APPROXIMATION_DUAL
+} AutomaticApproximation;
+
 ///Message stages
 #define MESSAGE_STAGE_CONVERSION		-4
 #define MESSAGE_STAGE_CONVERSION_PARSING	-3
@@ -153,9 +167,10 @@ typedef enum {
 #define MESSAGE_STAGE_UNSET			0
 
 ///Message categories
-#define MESSAGE_CATEGORY_NONE		0
-#define MESSAGE_CATEGORY_PARSING	1
-#define MESSAGE_CATEGORY_WIDE_INTERVAL	10
+#define MESSAGE_CATEGORY_NONE				0
+#define MESSAGE_CATEGORY_PARSING			1
+#define MESSAGE_CATEGORY_WIDE_INTERVAL			10
+#define MESSAGE_CATEGORY_IMPLICIT_MULTIPLICATION	11
 
 
 /// A message with information to the user. Primarily used for errors and warnings.
@@ -185,7 +200,10 @@ enum {
 	VARIABLE_ID_Y = 301,
 	VARIABLE_ID_Z = 302,
 	VARIABLE_ID_N = 303,
-	VARIABLE_ID_C = 304
+	VARIABLE_ID_C = 304,
+	VARIABLE_ID_PERCENT = 350,
+	VARIABLE_ID_PERMILLE = 351,
+	VARIABLE_ID_PERMYRIAD = 352
 };
 
 enum {
@@ -197,7 +215,11 @@ enum {
 	UNIT_ID_HOUR = 552,
 	UNIT_ID_DAY = 553,
 	UNIT_ID_MONTH = 554,
-	UNIT_ID_YEAR = 555
+	UNIT_ID_YEAR = 555,
+	UNIT_ID_KELVIN = 560,
+	UNIT_ID_RANKINE = 561,
+	UNIT_ID_CELSIUS = 562,
+	UNIT_ID_FAHRENHEIT = 563
 };
 
 enum {
@@ -215,6 +237,12 @@ enum {
 	TRANSACTINIDES
 };
 
+typedef enum {
+	TEMPERATURE_CALCULATION_HYBRID,
+	TEMPERATURE_CALCULATION_ABSOLUTE,
+	TEMPERATURE_CALCULATION_RELATIVE
+} TemperatureCalculationMode;
+
 struct Element {
 	std::string symbol, name;
 	int number, group;
@@ -348,12 +376,12 @@ class Calculator {
 	DecimalPrefix *decimal_null_prefix;
 	BinaryPrefix *binary_null_prefix;
 
-  	bool place_currency_code_before, place_currency_sign_before;
+	bool place_currency_code_before, place_currency_sign_before;
 	bool place_currency_code_before_negative, place_currency_sign_before_negative;
 	bool default_dot_as_separator;
 	std::string local_digit_group_separator, local_digit_group_format;
 
-  	bool b_busy;
+	bool b_busy;
 	std::string expression_to_calculate;
 	EvaluationOptions tmp_evaluationoptions;
 	MathStructure *tmp_parsedstruct;
@@ -392,7 +420,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
 	bool calculate(MathStructure *mstruct, std::string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -402,7 +430,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns The result of the calculation.
 	*/
 	MathStructure calculate(std::string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -427,7 +455,7 @@ class Calculator {
 	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options, std::string to_str = "");
 	/** Calculates an expression.and outputs the result to a text string. The expression should be unlocalized first with unlocalizeExpression().
 	*
-	* Unlike other functions for expression evaluation this function handles ending "to"-commmands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
+	* Unlike other functions for expression evaluation this function handles ending "to"-commands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
 	*
 	*
 	* @param str Expression.
@@ -439,6 +467,8 @@ class Calculator {
 	*/
 	std::string calculateAndPrint(std::string str, int msecs = 10000, const EvaluationOptions &eo = default_user_evaluation_options, const PrintOptions &po = default_print_options);
 	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, std::string *parsed_expression);
+	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, AutomaticFractionFormat auto_fraction, AutomaticApproximation auto_approx = AUTOMATIC_APPROXIMATION_OFF, std::string *parsed_expression = NULL, int max_length = -1, bool *result_is_comparison = NULL);
+	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, AutomaticFractionFormat auto_fraction, AutomaticApproximation auto_approx, std::string *parsed_expression, int max_length, bool *result_is_comparison, bool format, int colorize = 0, int tagtype = TAG_TYPE_HTML);
 	int testCondition(std::string expression);
 	//@}
 
@@ -446,7 +476,8 @@ class Calculator {
 	//@{
 	/** Calls MathStructure::format(po) and MathStructure::print(po). The process is aborted after msecs milliseconds.
 	*/
-	std::string print(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
+	std::string print(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &po = default_print_options);
+	std::string print(const MathStructure &mstruct, int milliseconds, const PrintOptions &po, bool format, int colorize = 0, int tagtype = TAG_TYPE_HTML);
 	///Deprecated: use print() instead
 	std::string printMathStructureTimeOut(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
 
@@ -469,6 +500,9 @@ class Calculator {
 	std::string abortedMessage(void) const;
 	std::string timedOutString(void) const;
 
+	std::string logicalORString(void) const;
+	std::string logicalANDString(void) const;
+
 	///Deprecated: use startControl()
 	void startPrintControl(int milliseconds = 0);
 	///Deprecated: use abort()
@@ -610,7 +644,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
 	bool RPNStackEnter(std::string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -626,7 +660,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	*/
 	void RPNStackEnter(std::string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	bool setRPNRegister(size_t index, MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_user_evaluation_options);
@@ -687,7 +721,7 @@ class Calculator {
 	bool parseAdd(std::string &str, MathStructure *mstruct, const ParseOptions &po);
 	//@}
 
-	/** @name Functions converting epxressions between units. */
+	/** @name Functions converting expressions between units. */
 	//@{
 	/** Converts to a unit expression.
 	* The converted value is evaluated.
@@ -695,9 +729,10 @@ class Calculator {
 	* @param mstruct The value to convert.
 	* @param composite_ Unit expression.
 	* @param eo Evaluation options.
-	* @param[out] units NULL or a math structure to fill with the parsed unit expression(or set to undefined if no units were found).
+	* @param[out] units NULL or a math structure to fill with the parsed unit expression (or set to undefined if no units were found).
 	* @returns Converted value.
 	*/
+	MathStructure convert(const MathStructure &mstruct, std::string composite_, const EvaluationOptions &eo, MathStructure *units, bool transform_orig, MathStructure *parsed_struct = NULL);
 	MathStructure convert(const MathStructure &mstruct, std::string composite_, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *units = NULL);
 	/** Converts to a unit.
 	* The converted value is evaluated.
@@ -708,11 +743,12 @@ class Calculator {
 	* @param always_convert ...
 	* @returns Converted value.
 	*/
+	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units, bool transform_orig, MathStructure *parsed_struct = NULL);
 	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
 	MathStructure convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(std::string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
-	/** Depecated: use convert() */
+	/** Deprecated: use convert() */
 	MathStructure convertTimeOut(std::string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(std::string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options);
@@ -726,6 +762,9 @@ class Calculator {
 	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options);
 	//@}
 
+	void setTemperatureCalculationMode(TemperatureCalculationMode temperature_calculation_mode);
+	TemperatureCalculationMode getTemperatureCalculationMode() const;
+
 	/** Used by the UI to find unit category for a mathematical expression.*/
 	Unit *findMatchingUnit(const MathStructure &mstruct);
 
@@ -786,7 +825,7 @@ class Calculator {
 	*
 	* @param exp10 Base-10 exponent of the value.
 	* @param exp The exponent of the unit.
-	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deka, hekto) will be skipped.
+	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deca, hecto) will be skipped.
 	* @returns A prefix or NULL if the unit should be left without prefix.
 	*/
 	DecimalPrefix *getOptimalDecimalPrefix(int exp10, int exp = 1, bool all_prefixes = true) const;
@@ -794,7 +833,7 @@ class Calculator {
 	*
 	* @param exp10 Base-10 exponent of the value.
 	* @param exp The exponent of the unit.
-	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deka, hekto) will be skipped.
+	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deca, hecto) will be skipped.
 	* @returns A prefix or NULL if the unit should be left without prefix.
 	*/
 	DecimalPrefix *getOptimalDecimalPrefix(const Number &exp10, const Number &exp, bool all_prefixes = true) const;
@@ -1079,6 +1118,10 @@ class Calculator {
 	int saveUnits(const char *file_name, bool save_global = false);
 	int saveFunctions(const char *file_name, bool save_global = false);
 	int saveDataSets(const char *file_name, bool save_global = false);
+	std::string saveTemporaryDefinitions();
+	void saveVariables(void *xmldoc, bool save_global = false, bool save_only_temp = false);
+	void saveUnits(void *xmldoc, bool save_global = false, bool save_only_temp = false);
+	void saveFunctions(void *xmldoc, bool save_global = false, bool save_only_temp = false);
 	//@}
 
 	/** @name Functions for CSV file import/export. */
@@ -1126,7 +1169,7 @@ class Calculator {
 	bool fetchExchangeRates(int seconds, std::string wget_args);
 	/** Download current exchange rates from the Internet to local disc with default wget arguments.
 	*
-	* @param seconds Maximum time for donwload try
+	* @param seconds Maximum time for download try
 	* @returns true if operation was successful.
 	*/
 	bool fetchExchangeRates(int seconds = 15, int n = -1);
@@ -1191,6 +1234,9 @@ class Calculator {
 	void endTemporaryEnableIntervalArithmetic();
 	//@}
 
+	bool usesMatlabStyleMatrices() const;
+	void useMatlabStyleMatrices(bool use_matlab_style_matrices);
+
 	void setCustomInputBase(Number nr);
 	void setCustomOutputBase(Number nr);
 	const Number &customInputBase() const;
